# Create a simple four node topology:
#
#        s1
#         \ 
#  8Mb,5ms \  0.8Mb,100ms
#           r1 --------- k1
#  8Mb,5ms /
#         /
#        s2
#
proc create_testnet { } {

	global s1 s2 r1 k1 n1 n2
	set s1 [ns node]
	set s2 [ns node]
	set k1 [ns node]
	#set r1 [ns node]

	create_test_atmnet

	ns_duplex $s1 $r1 8Mb 5ms drop-tail
	ns_duplex $s2 $r1 8Mb 5ms drop-tail
	set L [ns_duplex $r1 $k1 800kb 100ms drop-tail]
	[lindex $L 0] set queue-limit 6
	[lindex $L 1] set queue-limit 6
}


proc create_test_atmnet { } {
	global r1 n1 n2 s1 s2 k1 a1 a2
	set r1 [ns node atm-network min-hop]
	set n1 [ns node atm-switch output early]
	set n2 [ns node atm-switch shared simple]
	set a1 [ns node atm-source]
	set a2 [ns node atm-source]

	$n1 buffer-size 16
	$n2 buffer-size 36
	$r1 add-node $n1
	$r1 add-node $n2
	$r1 add-node $a1
	$r1 add-node $a2

	ns_duplex $n1 $n2 8Mb 10ms atm
	ns_duplex $a1 $n1 8Mb 10ms atm
	ns_duplex $a2 $n2 8Mb 10ms atm

	ns connect $s1 $r1 $n1
	ns connect $s2 $r1 $n2
	ns connect $k1 $r1 $n2
}

proc finish file {

	set f [open temp.rands w]
	puts $f "TitleText: $file"
	puts $f "Device: Postscript"
	
	exec rm -f temp.p temp.d 
	exec touch temp.d temp.p
	#
	# split queue/drop events into two separate files.
	# we don't bother checking for the link we're interested in
	# since we know only such events are in our trace file
	#
	exec awk {
		{
			if (($1 == "+" || $1 == "-" ) && \
			    ($5 == "tcp" || $5 == "ack"))\
					print $2, $8 + ($11 % 90) * 0.01
		}
	} out.tr > temp.p
	exec awk {
		{
			if ($1 == "d")
				print $2, $8 + ($11 % 90) * 0.01
		}
	} out.tr > temp.d

	puts $f \"packets
	flush $f
	exec cat temp.p >@ $f
	flush $f
	# insert dummy data sets so we get X's for marks in data-set 4
	puts $f [format "\n\"skip-1\n0 1\n\n\"skip-2\n0 1\n\n"]

	puts $f \"drops
	flush $f
	#
	# Repeat the first line twice in the drops file because
	# often we have only one drop and xgraph won't print marks
	# for data sets with only one point.
	#
	exec head -1 temp.d >@ $f
	exec cat temp.d >@ $f
	close $f
	exec xgraph -bb -tk -nl -m -x time -y packet temp.rands &
	exec ./stats 2 2 sink ouf 200
	exec ./stats 2 2 sink2 ouf2 200
	exec xgraph -x time -y Frequency ouf ouf2 &
	
	puts calls=[expr [$a1 calls-lost] [$a1 cals-accepted]]
	puts lost=[$a1 calls-lost]
	puts calls=[expr [$a2 calls-lost] [$a2 cals-accepted]]
	puts lost=[$a2 calls-lost]
	exit 0
}

#
# Arrange for tcp source stats to be dumped for $tcpSrc every
# $interval seconds of simulation time
#
proc tcpDump { tcpSrc interval } {
	proc dump { src interval } {
		global n1
		ns at [expr [ns now] + $interval] "dump $src $interval"
		puts [ns now]/cwnd=[$src get cwnd]/ssthresh=[$src get ssthresh]/ack=[$src get ack]
	}
	ns at 0.0 "dump $tcpSrc $interval"
}

proc tcpDumpAll { tcpSrc interval label } {
	proc dump { src interval label } {
		ns at [expr [ns now] + $interval] "dump $src $interval $label"
		puts $label/time=[ns now]/cwnd=[$src get cwnd]/ssthresh=[$src get ssthresh]/ack=[$src get ack]/rtt=[$src get rtt]
	}
	puts $label:window=[$tcpSrc get window]/packet-size=[$tcpSrc get packet-size]/bug-fix=[$tcpSrc get bug-fix]
	ns at 0.0 "dump $tcpSrc $interval $label"
}

proc openTrace { stopTime testName } {
	exec rm -f out.tr temp.rands
	global r1 k1    
	set traceFile [open out.tr w]
	ns at $stopTime \
		"close $traceFile ; finish $testName"
	set T [ns trace]
	$T attach $traceFile
	return $T
}

proc atmDump { switch interval port} {
	proc atmdump { switch interval port passed} {
		ns at [expr [ns now] + $interval] "atmdump $switch $interval $port [$switch passed]"
		puts time=[ns now]loss=[$switch loss]passed=[$switch passed]/throughput(cells/sec)=[expr ([$switch passed]-$passed)/$interval] 
	}
	ns at 1.0 "$switch reset"
	ns at 1.0 "atmdump $switch $interval $port 0"
}

proc audio_cbr { start end t1 t2 tr } {
	set qos_cbr [ns_atm_cbr_qos 640000 0.00001 1]
	set cbr_conn [ns_create_atm_cbr $t1 $t2]
	set cbr_src [lindex $cbr_conn 0] 
	$cbr_src qos $qos_cbr
	ns at $start "$cbr_src start"
	ns at $end "$cbr_src stop"

	[lindex $cbr_conn 1] trace $tr

}
proc finish_atm {} {
	global a1 a2
	exec ./stats 2 2 sink1 ou1 100
	exec ./stats 2 2 sink2 ou2 100
	exec ./stats 2 2 sink3 ou3 100
	exec ./stats 2 2 sink4 ou4 100
	puts calls=[expr [$a1 calls-lost]+ [$a1 calls-accepted]]
	puts lost=[$a1 calls-lost]
	puts calls=[expr [$a2 calls-lost]+ [$a2 calls-accepted]]
	puts lost=[$a2 calls-lost]
	exec xgraph -x time -y Frequency ou1 ou2 ou3 ou4 &
	exit 0
}
proc test_tahoe1 {} {
	global s1 s2 r1 k1 n1 n2 a1 a2
	create_testnet

	audio_cbr 1.0 4.0 $a1 $a2 sink1
	audio_cbr 3.0 8.0 $a1 $a2 sink2
	audio_cbr 2.5 5.0 $a2 $a1 sink3
	audio_cbr 8.0 14.0 $a2 $a1 sink4

	#set qos_cbr [ns_atm_cbr_qos 3000000 0.00001 1]
	#set cbr_conn [ns_create_atm_cbr $a1 $a2]
	
	#set qos_cbr2 [ns_atm_cbr_qos 9000000 0.00001 1]
	#set cbr_conn2 [ns_create_atm_cbr $a2 $a1]

	#set cbr_src [lindex $cbr_conn 0] 
	#$cbr_src qos $qos_cbr
	#ns at 1.0 "$cbr_src start"
	#ns at 6.0 "$cbr_src stop"

	#[lindex $cbr_conn 1] trace sink

	#set cbr_src2 [lindex $cbr_conn2 0] 
	#$cbr_src2 qos $qos_cbr2
	#ns at 1.00001 "$cbr_src2 start"
	#ns at 10.0 "$cbr_src2 stop"

	#[lindex $cbr_conn2 1] trace sink2


	ns at 11 "finish_atm"

	#set tcp1 [ns_create_connection tcp $s1 tcp-sink $k1 0]
	#$tcp1 set window 50
	#set ftp1 [$tcp1 source ftp]
	#ns at 0.0 "$ftp1 start"

	#tcpDump $tcp1 1.0
	atmDump $n2 1 0
	# trace only the bottleneck link
	#[ns link $r1 $k1] trace [openTrace 15.0 test_tahoe]

	ns run
}
