#include <stdio.h>
#include <stdlib.h>

#include "atmdiscard.h"
#include "atm_switches.h"

/*
 * Matcher classes for identifying the switches. 
 * "ns node atm-switch output $discard-algo" creates an output buffer switch
 * "ns node atm-switch shared $discard-algo" creates a shared memory switch
 */
static class ATMSwitchMatcher : public Matcher {
public:
        ATMSwitchMatcher() : Matcher("atm-switch") {}
        TclObject* match(const char* id) {
		if(strcasecmp(id, "output") == 0)
                	return (new ATMOutputBuffer());
		if(strcasecmp(id, "shared") == 0)
                	return (new ATMSharedBuffer());
		return (0);
        }
} matcher_atm_switch;

/*
 * The buffer_size is equally divided across all ports. Number of ports
 * includes both the atmlinks and the dummy external links.
 */
inline void ATMOutputBuffer::set_buffer_limit()
{
	buffer->qlimit_=buffer_size_/(no_links_ + dummy_count);
}

/*
 * The buffer_size is the queue limit.
 */
inline void ATMSharedBuffer::set_buffer_limit()
{
	buffer->qlimit_=buffer_size_;
}
