#include <stdlib.h>
#include <ctype.h>

#include "atmagent.h"
#include "atmsrc.h"

static class ATMQOSMatcher : public Matcher {
public:
        ATMQOSMatcher() : Matcher("qos") {}
        TclObject* match(const char* id) {
                return (new QOS);
        }
} matcher_atm_qos;


QOS::QOS()
{
	PCR=SCR=MCR=CDVT=CLR=CTD=0.0;
	link_real("ns_atm_qos", "PCR", &PCR, 0);
	link_real("ns_atm_qos", "SCR", &SCR, 0);
	link_real("ns_atm_qos", "CLR", &CLR, 0);
	link_real("ns_atm_qos", "CDVT", &CDVT, 0);
}

/*
 * $qos PCR $pcr
 * $qos SCR $scr
 * $qos MCR $mcr
 * $qos CDVT $cdvt
 * $qos CLR $clr
 * $qos CTD $ctd
 */
int QOS::command(int argc, const char*const* argv)
{
	if (argc == 3){
		double value=atof(argv[2]);
		if(strcmp(argv[1], "PCR")==0)
		{
			PCR=value;
			return (TCL_OK);
		}
		if(strcmp(argv[1], "SCR")==0)
		{
			SCR=value;
			return (TCL_OK);
		}
		if(strcmp(argv[1], "MCR")==0)
		{
			MCR=value;
			return (TCL_OK);
		}
		if(strcmp(argv[1], "CDVT")==0)
		{
			CDVT=value;
			return (TCL_OK);
		}
		if(strcmp(argv[1], "CLR")==0)
		{
			CLR=value;
			return (TCL_OK);
		}
		if(strcmp(argv[1], "CTD")==0)
		{
			CTD=value;
			return (TCL_OK);
		}
	}
	return (TclObject::command(argc, argv));
}
ATMAgent::ATMAgent():Agent(0)
{
	trace_=(0);
	qos_=(0);
	seqno_=(0);
}
/*
 * $agent qos $QOS
 * $agent trace $filename
 * $agent node $node
 * $agent start
 * $agent stop
 */
int ATMAgent::command(int argc, const char*const* argv)
{
        Tcl& tcl = Tcl::instance();
	if (argc ==2 ) {
                if (strcmp(argv[1], "start") == 0) {
			start();
			return (TCL_OK);
		}
                if (strcmp(argv[1], "stop") == 0) {
			stop();
			return (TCL_OK);
		}
	}
        if (argc == 3) {
                if (strcmp(argv[1], "qos") == 0) {
			qos_ = (QOS*)TclObject::lookup(argv[2]);
			if (! qos_ ){
				tcl.result("Unable to find QOS");
				return (TCL_ERROR);
			}
                        return (TCL_OK);
                }
                if (strcmp(argv[1], "node") == 0) {
			atmnode_ = (ATMSource*)TclObject::lookup(argv[2]);
			sport_ = atmnode_->add_agent(this);
			return (TCL_OK);
		}
                if (strcmp(argv[1], "trace") == 0) {
			trace_ = new ATMTrace(argv[2]);
			if (! trace_ ){
				tcl.result("Unable to Create Trace");
				return (TCL_ERROR);
			}
			return (TCL_OK);
		}
		
	}
        if (argc == 2) {
                if (strcmp(argv[1], "addr") == 0) {
                        char* bp = tcl.buffer();
                        sprintf(bp, "%d", atmnode_->addr());
                        tcl.result(bp);
                        return (TCL_OK);
                }
                if (strcmp(argv[1], "node") == 0) {
                        tcl.result(atmnode_->name());
                        return (TCL_OK);
                }
        }
        else if (argc == 4) {
                if (strcmp(argv[1], "connect") == 0) {
                        daddr_ = strtol(argv[2], (char**)0, 0);
                        dport_ = strtol(argv[3], (char**)0, 0);
                        if (daddr_ == atmnode_->addr()) {
                                tcl.result("ns connect: can't connect to self");
                                return (TCL_ERROR);
                        }
                        return (TCL_OK);
                }
        }
        return (Agent::command(argc, argv));
}
