#include "atmcbr.h"
#include "atmsrc.h"
#include "atmnetwork.h"

static class ATMCBRSourceMatcher : public Matcher {
public:
        ATMCBRSourceMatcher() : Matcher("agent") {}
        TclObject* match(const char* id) {
                if (strcasecmp(id, "atm-cbr-src") == 0)
                        return (new ATMCBRSource);
                return (0);
        }
} matcher_atm_cbr_src;

static class ATMCBRSinkMatcher : public Matcher {
public:
        ATMCBRSinkMatcher() : Matcher("agent") {}
        TclObject* match(const char* id) {
                if (strcasecmp(id, "atm-cbr-sink") == 0)
                        return (new ATMCBRSink);
                return (0);
        }
} matcher_atm_cbr_sink;


ATMCell* ATMCBRSource::generate_cell()
{
	ATMCell * cell=new ATMCell;
	cell->qtime_=(Scheduler::instance()).clock();	//Generation time
	cell->source_=atmnode_->addr();
	cell->dest_ =daddr_;
	cell->sport_=sport_;
	cell->dport_=dport_;
	cell->atmseq_no_=seqno_++;
	cell->seqno_=-1;
	cell->queued_=0;
	cell->priority_=1;
	cell->total_cells_=1;
	return cell;
}

void ATMCBRSource::handle(Event* e)
{
	if(status==0)
		return;		//Ignore all events
	Scheduler& s = Scheduler::instance();
	s.schedule(atmnode_, e , 0 );
	ATMCell* cell=generate_cell();
	s.schedule(this, cell , 1.0/(qos_->SCR) );
}
void ATMCBRSource::start()
{
	if((atmnode_->network())->CAC(qos_, atmnode_->addr(), daddr_))
	{
		ATMCell* cell=generate_cell();
		Scheduler& s = Scheduler::instance();
		s.schedule(this, cell , 0 );
		status=1;
		atmnode_->Accepted();
	}
	else
	{
		Tcl& tcl = Tcl::instance();
		tcl.result("Unable to establish Connection");
		atmnode_->Refused();
	}
}
void ATMCBRSource::stop()
{
	status=0;
	(atmnode_->network())->deallocate_resources(qos_, 
			atmnode_->addr(), daddr_);
}

void ATMCBRSink::handle(Event* e)
{
	Scheduler & s=Scheduler::instance();
	if(!trace_)
		return;	
	ATMCell* cell=(ATMCell*)e;
	char line[100];
	double RTT=s.clock()-cell->qtime_;
	sprintf(line, "%d\t%.6f\n", cell->atmseq_no_ , RTT);
	trace_->dump(line);
}
void ATMCBRSink::start()
{
}
void ATMCBRSink::stop()
{
	delete trace_;
	trace_=(0);
}
