/*
#include "scheduler.h"
#include "object.h"
#include "atmcell.h"
*/
#include "atmdiscard.h"
#include "atmswitch.h"

/*
 * These are used to identify the discard algorithm. 
 * "output" specifies that it is for output buffer
 * and "simple" and "early" identify the discard algorithm
 */

static class ATMOutputDiscardMatcher : public Matcher {
public:
        ATMOutputDiscardMatcher() : Matcher("output") {}
        TclObject* match(const char* id) {
                if (strcasecmp(id, "simple") == 0)
                        return (new SimpleOutputBuffer());
                if (strcasecmp(id, "early") == 0)
                        return (new EarlyOutputBuffer);
                return (0);
        }
} matcher_atm_output_discard;

static class ATMSharedDiscardMatcher : public Matcher {
public:
        ATMSharedDiscardMatcher() : Matcher("shared") {}
        TclObject* match(const char* id) {
                if (strcasecmp(id, "simple") == 0)
                        return (new SimpleSharedBuffer);
                if (strcasecmp(id, "early") == 0)
                        return (new EarlySharedBuffer);
                return (0);
        }
} matcher_atm_shared_discard;

/*
 * When the individual port queue is full then discard
 */
int SimpleOutputBuffer::check_discard(const ATMSwitchBuffer* buffer, 
					const ATMCell* cell, int port) 
{
        if(buffer->qsize_[port] >= buffer->qlimit_)
                return 1;
        return 0;
}

/*
 * When the combined total of the port queues is full then discard
 */
int SimpleSharedBuffer::check_discard(const ATMSwitchBuffer* buffer, 
					const ATMCell* cell, int port)
{
        if(buffer->total_q_ >= buffer->qlimit_)
                return 1;
        return 0;

}

/*
 * Set weight to the default weight value
 */
EarlyOutputBuffer::EarlyOutputBuffer()
{
	link_real("ns_early_discard", "weight", &weight_, 0);
}

/*
 * If a percentage(weight) of the queue is full then discard UBR cells
 * else if all the queue is full then discard all
 * However look at the queue at only the specified port.
 */
int EarlyOutputBuffer::check_discard(const ATMSwitchBuffer* buffer, 
					const ATMCell* cell, int port)
{
        if(buffer->qsize_[port] >= buffer->qlimit_)
                return 1;
        if(cell->priority_)		//Not UBR
		return 0;
	if(buffer->qsize_[port] >= weight_*buffer->qlimit_)
                return 1;
        return 0;
}

EarlySharedBuffer::EarlySharedBuffer()
{
	link_real("ns_early_discard", "weight", &weight_, 0);
}

/*
 * If a percentage(weight) of the queue is full then discard UBR cells
 * else if all the queue is full then discard all
 * However look at the whole buffer queue 
 */
int EarlySharedBuffer::check_discard(const ATMSwitchBuffer* buffer,
			const ATMCell* cell, int port)
{
        if(buffer->total_q_ >= buffer->qlimit_)
                return 1;
        if(cell->priority_)		//Not UBR
		return 0;
	if(buffer->total_q_ >= weight_*buffer->qlimit_)
                return 1;
        return 0;
}
