#include <string.h>
#include "atmlink.h"

/*
 * Matcher class for identifying the ATMLink. The type "atm" identifies it to
 * be an ATMLink
 */
static class ATMLinkMatcher : public Matcher {
public:
        ATMLinkMatcher() : Matcher("link") {}
        TclObject* match(const char* id) {
                if (strcasecmp(id, "atm") == 0)
                        return (new ATMLink);
                return (0);
        }
} matcher_atm_link;

ATMLink::ATMLink()
{
	neighbor_=(0);
	start=0.0;
	used_bw_=(0);
}

/*
 * Set the neighboring node and the destination address
 */
inline void ATMLink::neighbor(ATMNode* p)
{ 
	neighbor_ = p; 
	dst_ = p->addr(); 
}

/*
 * Find the slotsize and then schedule the time when the cell will be 
 * reached at the other end.
 */
void ATMLink::handle(Event* e)
{
	ATMCell * cell=(ATMCell*)e;
        if (cell != 0) {
                double slotsize = 424. / bandwidth_;
                Scheduler& s = Scheduler::instance();
		double curr_time = s.clock();
		if(curr_time < start)
			curr_time=start;
		curr_time+=slotsize;
	        double next_slot=curr_time/slotsize;
        	if(next_slot-(long int)next_slot>0)
                	next_slot=(long int) next_slot+1;
		start=next_slot*slotsize;
        	next_slot=start-s.clock() ;
		
                ofstream fout("output",ios::app);
		fout<<"\n"<<s.clock()<<"\tPassing Link\t"<<cell;
			/*->seqno_<<" ("
			<<cell->atmseq_no_<<","<<cell->total_cells_<<")\t"
			<<cell->source_<<" -> "<<cell->dest_;*/
                fout.close(); 

                s.schedule(neighbor_, cell , next_slot + delay_);
                busy_ = 1;
                busytime_ += slotsize;
                if (trace_ != 0)
                        trace_->hop(src_, dst_, cell);
        }

}

/*
 * Only override the "install" command to be between two ATMNodes and then
 * add it to the network also.
 */
int ATMLink::command(int argc, const char*const* argv)
{
	Tcl& tcl = Tcl::instance();
	if (argc == 4) {
		if (strcmp(argv[1], "install") == 0) {
			ATMNode* src = (ATMNode*)TclObject::lookup(argv[2]);
			ATMNode* dst = (ATMNode*)TclObject::lookup(argv[3]);

			//check that both the ends are atmnodes
			if (src == dst) {
				Tcl& tcl = Tcl::instance();
				tcl.resultf("link install: self-loop");
				return (TCL_ERROR);
			}
			neighbor(dst);
			src->add_link(this);
			return (TCL_OK);
		}
	}
	//Otherwise let the main Link handle it
	return (Link::command(argc, argv));
}
