#ifndef atm_link_h
#define atm_link_h

#include "scheduler.h"
#include "object.h"
#include "integrator.h"
#include "atmcell.h"
#include "trace.h"
#include "flow.h"
#include "link.h"

class ATMNode;

/*
 * The ATMLink is just like an ordinary link but the difference is that the end
 * nodes are ATMNodes and the transmission time is fixed i.e. cell size is fixed
 * Also the links are synchronized at time 0.0 i.e. a cell can enter from any
 * end at multiples of the slottime. 
 */
class ATMLink : public Link {
 public:
	ATMLink();
	void send(ATMCell*);
	void handle(Event*);	//Handles the events
	
	inline double bandwidth(){return bandwidth_;};

        inline ATMNode* neighbor() const { return neighbor_; }
        inline void neighbor(ATMNode* p);
	inline double available_bw(){return (bandwidth_ - used_bw_);};
	inline void allocate_bw(double b){used_bw_+=b;};
	inline void deallocate_bw(double b){used_bw_-=b;};

	int command(int argc, const char*const* argv);

	//Just to stop the compiler from complaining. Should Never be called
        inline int qlen() { return(0); }

 protected:
	ATMNode* neighbor_;	//The end node for the link
	double start;		//The start time for when the next cell can go.
	double used_bw_;		//used bandwidth

	//Just to stop the compiler from complaining. Should Never be called
	void enque(Packet* p){exit(0);};	
	Packet* deque(){exit(0);};	
};
#include "atmnode.h"
#endif
