#include "object.h"
#include "scheduler.h"
#include "node.h"
#include "atmnode.h"
#include "atmroute.h"

#ifndef ns_atmnetwork_h
#define ns_atmnetwork_h

class QOS;
/*
 * This class represents a complete ATM Network. It is derived from the Node 
 * class so that the current TCP remains unaffected while it can internally
 * handle packets the way it wants to.
*/

class ATMNetwork: public Node {
  protected:
        ATMNode** nodes;	//All switches and ATM source nodes
	int* map;		//Used for identifying the external TCP
			//connections. map[tcpnode address]->ATMNode index
	double* earliest_start; 
		//Earliest time a packet can arrive from any link.
	int curr_nodes;		//current number of nodes
	int max_nodes;		//MaxSize of nodes. Doubles whenever full
			//so that the number of nodes in a network is only
			//constrained by memory.
	
	ATMLink *** link_table;	//The ATMLinks in the simulator
			//link_table[src][dest] -> gives the link

	ATMRouteLogic* routelogic; //The routing logic followed
 public:
	ATMNetwork();		
	~ATMNetwork();

	void handle(Event*);	//Event Handler
	virtual int command(int argc, const char*const* argv);
				//Handles input Tcl commands

	int attach(Node* , ATMNode *);	//Attach External TcpNode to ATMNode
        int lookup_id(const char*);	//Search for node inside the network
	void add_node(ATMNode * );	//Add a node to the network
	void add_link(ATMLink* , int,int);	//Add a link
	Link** route(){return route_; };	//TCP route
	inline int dummy_addr(int d){return ((route_[d])->neighbor())->addr();};
			//Given the destination, returns the address of the 
			//next TCP node. 

	int CAC(QOS* , int,int);
	void deallocate_resources(QOS * , int , int );
};
#endif
