#include <string.h>
#include "atmnode.h"
#include "atmnetwork.h"

/*
 * The ATMNode constructor. The number of ATMLinks is initially 16 and will 
 * double whenever needed. 
 */
ATMNode::ATMNode()
{
	links_=new ATMLink*[16];
	port_table=new int[16];
	no_links_=0;
	aal=NULL;
	total_nodes=16;
	dummy_=(0);
	dummy_count=0;
}
ATMNode::~ATMNode()
{
	if(aal!=NULL)
		delete aal;
}
ATMNode::ATMNode(int tn)
{
	total_nodes=tn;
	no_links_=0;
	links_=new ATMLink*[tn];
	port_table=new int[tn];
	aal=new ATMAal[tn];
}

/*
 * Initialise the aal layer. Called by the ATMNetwork whenever the number of
 * nodes changes
 */
void ATMNode::init_aal(int tn)
{
	total_nodes=tn;
	if(aal!=NULL)
		delete aal;
	aal=new ATMAal[tn];
}

/*
 * Add a dummy link to this node for the specified address. This is called
 * by the network when it connects a tcpnode to an atmnode.
 */
void ATMNode::add_dummy_link(int ad)
{
	DummyLinks* p=new DummyLinks;
	p->addr=ad;
	p->port=dummy_count++;	//Assign the next number
	p->next=dummy_;
	dummy_=p;
}

/*
 * This searches for the address in the dummy ports and then returns the 
 * port number of the queue
 */
int ATMNode::get_dummy_port(int ad)
{
	for(DummyLinks* p=dummy_;p!=(0);p=p->next)
		if(ad==p->addr)
			return p->port+no_links_;
	return -1;
}
/*
 * add a link to this node. If they have reached their maximum number then
 * double the list.
 */
void ATMNode::add_link(ATMLink* link)
{
	link->src(addr_);
	int des=(link->neighbor())->addr_;
	if(des>=total_nodes)
	{
                total_nodes*=2;
                ATMLink ** t_link=new ATMLink*[total_nodes];
		int * t_ports=new int[total_nodes];
                int i=0;
                for(;i<(total_nodes/2);i++)
                {
                        t_link[i]=links_[i];
			t_ports[i]=port_table[i];
                }
                for(;i<total_nodes;i++)
                {
                        t_link[i]=(0);
			t_ports[i]=0;
                }
                links_=t_link;
		port_table=t_ports;
        }
	links_[des]=link;
	network_->add_link(link,addr_,des);
	port_table[des]=no_links_;
	no_links_++;
}
