#include <stdio.h>
#include <stdlib.h>

#include "packet.h"
#include "atmsrc.h"
#include "node.h"
#include "link.h"
#include "agent.h"
#include "atmnetwork.h"

static class ATMSourceMatcher : public Matcher {
public:
        ATMSourceMatcher() : Matcher("atm-source") {}
        TclObject* match(const char* id) {
                return (new ATMSource());
        }
} matcher_atm_source;

ATMSource::ATMSource()
{
	nport_=0;
	maxport_=16;
	conn_lost=0;
	conn_accepted=0;
        agents_ = new ATMAgent*[maxport_];
        for (int i = 0; i < maxport_; ++i)
                agents_[i] = 0;
}

int ATMSource::add_agent(ATMAgent* p)
{
        int port = nport_++;
        if (port >= maxport_) {
                int n = maxport_ * 2;
                ATMAgent** temp= new ATMAgent*[n];
                int i;
                for (i = 0; i < maxport_; ++i)
                        temp[i] = agents_[i];
                for (; i < n; ++i)
                        temp[i] = 0;
                delete agents_;
                agents_ = temp;
                maxport_ = n;
        }
        agents_[port] = p;
        return (port);
}

void ATMSource::handle(Event* x)
{
	ATMCell* cell=(ATMCell*)x;
        Scheduler& s = Scheduler::instance();
	
	//if I am the destination then handover to the agent
	//else pass it on

	if(cell->dest_ == addr_)	
		s.schedule (agents_[cell->dport_],x,0);
	else
	//Just route the packet
                s.schedule(links_[cell->dest_], cell, 0);
}

/*
 * $src calls-lost
 * $src calls-accepted
 */
int ATMSource::command(int argc, const char*const* argv)
{
        Tcl& tcl = Tcl::instance();
	if (argc == 2)
	{
                if(strcmp(argv[1], "calls-lost")==0)
                {
                        char* bp = tcl.buffer();
                        sprintf(bp, "%d", conn_lost);
                        tcl.result(bp);
                        return (TCL_OK);
                }
                if(strcmp(argv[1], "calls-accepted")==0)
                {
                        char* bp = tcl.buffer();
                        sprintf(bp, "%d", conn_accepted);
                        tcl.result(bp);
                        return (TCL_OK);
                }
	}
	return (ATMNode::command(argc, argv));
}
