#ifndef atmswitch_h
#include "atmcell.h"
#include "atmnode.h"

/*
 * The buffer for a switch. Even the buffer can be inherited to form a different
 * buffers to represent different switching architectures. We do not need to 
 * store the actual cells. Only counters will be fine.
 */
class ATMSwitchBuffer {
public:
	double * exit_time;
	int * qsize_;		 //The current qsize
	int total_q_;		//Sum of all the queues
	int ports;		//The number of ports
	int qlimit_;		//The qlimit to compare with for discard
	ATMSwitchBuffer() : total_q_(0) , qsize_(0), qlimit_(0) {};
};

/*
 * Generic DiscardAlgorithm class. All discard algorithms can be implemented
 * by inheriting from this. Only have to make one function that looking at
 * the buffer and the cell determines whether the cell should be discarded or
 * not.
 */
class DiscardAlgorithm : public NsObject {
	public:
	virtual int check_discard(const ATMSwitchBuffer*,const ATMCell*,int)=0 ;
};

/*
 * The base class for all ATMSwitches. If 
 */

class ATMSwitch : public ATMNode{
	public:
		ATMSwitch();
                virtual void handle(Event* );
		virtual void init_aal(int);
		virtual void add_link(ATMLink*);
                virtual command(int argc, const char*const* argv);            
		int discard(int);

		//int check_resources(void);
		//int free_bandwidth( bandwidth,double link_bandwidth);
		//int reserve_bandwidth(int output,double link_bandwidth);
		//void show_statistics(ostream&);
 	protected:
		ATMSwitchBuffer* buffer;	//Buffer for the switch
		DiscardAlgorithm* discard_;	//Discard algorithm
		virtual void enque(ATMCell*,int);	//Enque cell
		virtual void deque(ATMCell*,int);	//Deque Cell
		virtual inline void set_buffer_limit()=0;	

		int buffer_size_;	//Buffer size in cells
		int qlimit;		//??? Needed
		long int no_of_cells_lost;	//Cells passed
		long int no_of_cells_passed;	//Cells lost
};
#define atmswitch_h
#endif
